@@ -14,8 +14,8 @@ from distributed.client import _wait
 from distributed.metrics import time
 from distributed.utils_test import gen_cluster, inc, dec
 from distributed.bokeh.worker import (BokehWorker, StateTable, CrossFilter,
-        CommunicatingStream, ExecutingTimeSeries, CommunicatingTimeSeries,
-        SystemMonitor, Counters)
+                                      CommunicatingStream, ExecutingTimeSeries, CommunicatingTimeSeries,
+                                      SystemMonitor, Counters)
 
 
 @pytest.mark.skipif(sys.version_info[0] == 2,
@@ -36,10 +36,17 @@ def test_simple(c, s, a, b):
         assert 'bokeh' in response.body.decode().lower()
 
 
+@gen_cluster(client=True,
+             worker_kwargs={'services': {('bokeh', 0):  (BokehWorker, {})}})
+def test_services_kwargs(c, s, a, b):
+    assert s.worker_info[a.address]['services'] == {'bokeh': a.services['bokeh'].port}
+    assert isinstance(a.services['bokeh'], BokehWorker)
+
+
 @gen_cluster(client=True)
 def test_basic(c, s, a, b):
     for component in [StateTable, ExecutingTimeSeries,
-            CommunicatingTimeSeries, CrossFilter, SystemMonitor]:
+                      CommunicatingTimeSeries, CrossFilter, SystemMonitor]:
 
         aa = component(a)
         bb = component(b)
@@ -49,7 +56,6 @@ def test_basic(c, s, a, b):
 
         def slowall(*args):
             sleep(1)
-            pass
 
         x = c.submit(slowall, xs, ys, 1, workers=a.address)
         y = c.submit(slowall, xs, ys, 2, workers=b.address)
@@ -78,7 +84,6 @@ def test_counters(c, s, a, b):
         yield gen.sleep(1)
         assert time() < start + 5
 
-
     a.digests['foo'].add(1)
     a.digests['foo'].add(2)
     aa.add_digest_figure('foo')
@@ -88,6 +93,11 @@ def test_counters(c, s, a, b):
     a.counters['bar'].add(2)
     aa.add_counter_figure('bar')
 
+    for x in [aa.counter_sources.values(), aa.digest_sources.values()]:
+        for y in x:
+            for z in y.values():
+                assert len(set(map(len, z.data.values()))) == 1
+
 
 @gen_cluster(client=True)
 def test_CommunicatingStream(c, s, a, b):
