@@ -14,7 +14,6 @@ For more information see: http://click.pocoo.org/5/python3/
 
 from distributed.comm import (parse_address, unparse_address,
                               parse_host_port, unparse_host_port)
-from ..utils import get_ip, ensure_ip
 
 
 def check_python_3():
@@ -36,7 +35,7 @@ def install_signal_handlers():
     import signal
 
     def handle_signal(sig, frame):
-        IOLoop.instance().add_callback(IOLoop.instance().stop)
+        IOLoop.instance().add_callback_from_signal(IOLoop.instance().stop)
 
     signal.signal(signal.SIGINT, handle_signal)
     signal.signal(signal.SIGTERM, handle_signal)
