@@ -1,18 +1,15 @@
 from __future__ import print_function, division, absolute_import
 
-from time import sleep
 
 import pytest
 pytest.importorskip('bokeh')
-from tornado import gen
 
-from dask import do
+from dask import delayed
 from distributed.client import _wait
 from distributed.diagnostics.progress_stream import (progress_quads,
-        nbytes_bar, progress_stream, _incrementing_index_cache)
-from distributed.metrics import time
-from distributed.utils_test import inc, div, dec, gen_cluster
-from distributed.worker import dumps_task
+                                                     nbytes_bar, progress_stream, _incrementing_index_cache)
+from distributed.utils_test import div, gen_cluster, inc
+
 
 def test_progress_quads():
     msg = {'all': {'inc': 5, 'dec': 1, 'add': 4},
@@ -58,7 +55,7 @@ def test_progress_stream(c, s, a, b):
 
     x = 1
     for i in range(5):
-        x = do(inc)(x)
+        x = delayed(inc)(x)
     future = c.compute(x)
 
     yield _wait(futures + [future])
