@@ -1,14 +1,11 @@
 from __future__ import print_function, division, absolute_import
 
 import json
-import sys
 
-from tornado.ioloop import IOLoop
-from tornado import web, gen
 from tornado.httpclient import AsyncHTTPClient
 from tornado.httpserver import HTTPServer
 
-from distributed import Scheduler, Client
+from distributed import Scheduler
 from distributed.client import _wait
 from distributed.sizeof import getsizeof
 from distributed.utils_test import gen_cluster, gen_test, inc, div
@@ -97,15 +94,23 @@ def test_broadcast(s, a, b):
 @gen_test()
 def test_services():
     s = Scheduler(services={'http': HTTPScheduler})
-    assert isinstance(s.services['http'], HTTPServer)
-    assert s.services['http'].port
+    s.start()
+    try:
+        assert isinstance(s.services['http'], HTTPServer)
+        assert s.services['http'].port > 0
+    finally:
+        s.close()
 
 
 @gen_test()
 def test_services_with_port():
     s = Scheduler(services={('http', 9999): HTTPScheduler})
-    assert isinstance(s.services['http'], HTTPServer)
-    assert s.services['http'].port == 9999
+    s.start()
+    try:
+        assert isinstance(s.services['http'], HTTPServer)
+        assert s.services['http'].port == 9999
+    finally:
+        s.close()
 
 
 @gen_cluster(client=True)
@@ -132,11 +137,11 @@ def test_with_data(e, s, a, b):
     out = json.loads(response.body.decode())
     assert set(out) == {a.address, b.address}
     assert all(isinstance(v, dict) for v in out.values())
-    assert all(k in {'inc', 'data'} for d in out.values() for k in d)
+    assert all(k in {'inc', 'str'} for d in out.values() for k in d)
     assert all(isinstance(v, int) for d in out.values() for v in d.values())
 
     assert sum(v for d in out.values() for v in d.values()) == \
-            sum(map(getsizeof, [1, 2, 3, 'Hello', 'world!']))
+        sum(map(getsizeof, [1, 2, 3, 'Hello', 'world!']))
 
     ss.stop()
 
@@ -171,7 +176,6 @@ def test_with_status(e, s, a, b):
     ss.stop()
 
 
-
 @gen_cluster(client=True)
 def test_basic(e, s, a, b):
     ss = HTTPScheduler(s)
