@@ -11,7 +11,7 @@ from tornado import gen
 from tornado.ioloop import IOLoop, TimeoutError
 from tornado.locks import Event
 
-from .comm import get_address_host, get_local_address_for
+from .comm import get_address_host, get_local_address_for, unparse_host_port
 from .core import rpc, RPCClosed, CommClosedError, coerce_to_address
 from .node import ServerNode
 from .process import AsyncProcess
@@ -36,7 +36,8 @@ class Nanny(ServerNode):
                  ncores=None, loop=None, local_dir=None, services=None,
                  name=None, memory_limit='auto', reconnect=True,
                  validate=False, quiet=False, resources=None, silence_logs=None,
-                 death_timeout=None, preload=(), security=None, **kwargs):
+                 death_timeout=None, preload=(), security=None,
+                 contact_address=None, listen_address=None, **kwargs):
         if scheduler_port is None:
             self.scheduler_addr = coerce_to_address(scheduler_ip)
         else:
@@ -48,6 +49,7 @@ class Nanny(ServerNode):
         self.resources = resources
         self.death_timeout = death_timeout
         self.preload = preload
+        self.contact_address = contact_address
 
         self.security = security or Security()
         assert isinstance(self.security, Security)
@@ -79,6 +81,7 @@ class Nanny(ServerNode):
                                     connection_args=self.connection_args,
                                     **kwargs)
 
+        self._listen_address = listen_address
         self.status = 'init'
 
     def __str__(self):
@@ -141,7 +144,7 @@ class Nanny(ServerNode):
         self.loop.add_callback(self._start, addr_or_port)
 
     @gen.coroutine
-    def kill(self, comm=None, timeout=10):
+    def kill(self, comm=None, timeout=2):
         """ Kill the local worker process
 
         Blocks until both the process is down and the scheduler is properly
@@ -152,7 +155,7 @@ class Nanny(ServerNode):
             raise gen.Return('OK')
 
         deadline = self.loop.time() + timeout
-        yield self.process.kill(grace_delay=0.8 * (deadline - self.loop.time()))
+        yield self.process.kill(timeout=0.8 * (deadline - self.loop.time()))
         yield self._unregister(deadline - self.loop.time())
 
     @gen.coroutine
@@ -161,6 +164,13 @@ class Nanny(ServerNode):
 
         Blocks until the process is up and the scheduler is properly informed
         """
+        if self._listen_address:
+            start_arg = self._listen_address
+        else:
+            host = self.listener.bound_address[0]
+            start_arg = self.listener.prefix + unparse_host_port(host,
+                    self._given_worker_port)
+
         if self.process is None:
             self.process = WorkerProcess(
                 worker_args=(self.scheduler_addr,),
@@ -176,8 +186,9 @@ class Nanny(ServerNode):
                                    silence_logs=self.silence_logs,
                                    death_timeout=self.death_timeout,
                                    preload=self.preload,
-                                   security=self.security),
-                worker_start_args=(self._given_worker_port,),
+                                   security=self.security,
+                                   contact_address=self.contact_address),
+                worker_start_args=(start_arg,),
                 silence_logs=self.silence_logs,
                 on_exit=self._on_exit,
             )
@@ -195,9 +206,9 @@ class Nanny(ServerNode):
         raise gen.Return('OK')
 
     @gen.coroutine
-    def restart(self, comm=None):
+    def restart(self, comm=None, timeout=2):
         if self.process is not None:
-            yield self.kill()
+            yield self.kill(timeout=timeout)
         yield self.instantiate()
         raise gen.Return('OK')
 
@@ -333,6 +344,7 @@ class WorkerProcess(object):
             self.status = 'stopped'
             self.stopped.set()
             # Release resources
+            self.process.close()
             self.init_result_q = None
             self.child_stop_q = None
             self.process = None
@@ -345,13 +357,13 @@ class WorkerProcess(object):
                 self.on_exit(r)
 
     @gen.coroutine
-    def kill(self, grace_delay=10):
+    def kill(self, timeout=2):
         """
         Ensure the worker process is stopped, waiting at most
-        *grace_delay* seconds before terminating it abruptly.
+        *timeout* seconds before terminating it abruptly.
         """
         loop = IOLoop.current()
-        deadline = loop.time() + grace_delay
+        deadline = loop.time() + timeout
 
         if self.status == 'stopped':
             return
@@ -371,7 +383,7 @@ class WorkerProcess(object):
 
         if process.is_alive():
             logger.warning("Worker process still alive after %d seconds, killing",
-                           grace_delay)
+                           timeout)
             try:
                 yield process.terminate()
             except Exception as e:
@@ -454,7 +466,7 @@ class WorkerProcess(object):
             try:
                 yield worker._start(*worker_start_args)
             except Exception as e:
-                logger.exception(e)
+                logger.exception("Failed to start worker")
                 init_result_q.put(e)
             else:
                 assert worker.address
