@@ -1,17 +1,15 @@
 from __future__ import print_function, division, absolute_import
 
-from operator import add
 from time import sleep
 import sys
 
 import pytest
-from toolz import take
 from tornado import gen
 
-from distributed import Client, Variable, worker_client, Nanny
+from distributed import Client, Variable, worker_client, Nanny, wait
 from distributed.metrics import time
-from distributed.utils_test import (gen_cluster, inc, loop, cluster, slowinc,
-                                    slow)
+from distributed.utils_test import (gen_cluster, inc, cluster, slow, div)
+from distributed.utils_test import loop # flake8: noqa
 
 
 @gen_cluster(client=True)
@@ -70,7 +68,7 @@ def test_hold_futures(s, a, b):
     x1 = Variable('x')
     yield x1.set(future)
     del x1
-    yield c1.shutdown()
+    yield c1.close()
 
     yield gen.sleep(0.1)
 
@@ -80,7 +78,7 @@ def test_hold_futures(s, a, b):
     result = yield future2
 
     assert result == 11
-    yield c2.shutdown()
+    yield c2.close()
 
 
 @gen_cluster(client=True)
@@ -171,4 +169,38 @@ def test_race(c, s, *workers):
     while len(s.wants_what['variable-x']) != 1:
         yield gen.sleep(0.01)
         if not time() - start < 2:
-            import pdb; pdb.set_trace()
+            import pdb
+            pdb.set_trace()
+
+
+@gen_cluster(client=True)
+def test_Future_knows_status_immediately(c, s, a, b):
+    x = yield c.scatter(123)
+    v = Variable('x')
+    yield v.set(x)
+
+    c2 = yield Client(s.address, asynchronous=True)
+    v2 = Variable('x', client=c2)
+    future = yield v2.get()
+    assert future.status == 'finished'
+
+    x = c.submit(div, 1, 0)
+    yield wait(x)
+    yield v.set(x)
+
+    future2 = yield v2.get()
+    assert future2.status == 'error'
+    with pytest.raises(Exception):
+        yield future2
+
+    start = time()
+    while True:  # we learn about the true error eventually
+        try:
+            yield future2
+        except ZeroDivisionError:
+            break
+        except Exception:
+            assert time() < start + 5
+            yield gen.sleep(0.05)
+
+    yield c2.close()
