@@ -2,35 +2,31 @@ from __future__ import print_function, division, absolute_import
 
 from concurrent.futures import ThreadPoolExecutor
 import logging
-from numbers import Integral, Number
+from numbers import Number
 from operator import add
 import os
-import re
 import shutil
 import sys
 import traceback
 
 from dask import delayed
 import pytest
-from toolz import pluck, sliding_window
+from toolz import pluck, sliding_window, first
 import tornado
 from tornado import gen
 from tornado.ioloop import TimeoutError
 
-import distributed
-from distributed import Nanny
-from distributed.core import rpc, connect
+from distributed import Nanny, Client, get_client, wait, default_client
+from distributed.compatibility import WINDOWS
+from distributed.core import rpc
 from distributed.client import _wait
 from distributed.scheduler import Scheduler
 from distributed.metrics import time
-from distributed.protocol import to_serialize
-from distributed.protocol.pickle import dumps, loads
-from distributed.sizeof import sizeof
 from distributed.worker import Worker, error_message, logger, TOTAL_MEMORY
-from distributed.utils import ignoring, tmpfile
-from distributed.utils_test import (loop, inc, mul, gen_cluster, div, dec,
-        slow, slowinc, throws, gen_test, readone)
-
+from distributed.utils import tmpfile
+from distributed.utils_test import (inc, mul, gen_cluster, div, dec,
+                                    slow, slowinc, gen_test, cluster)
+from distributed.utils_test import loop # flake8: noqa
 
 
 def test_worker_ncores():
@@ -85,8 +81,10 @@ def test_health():
 def test_worker_bad_args(c, s, a, b):
     class NoReprObj(object):
         """ This object cannot be properly represented as a string. """
+
         def __str__(self):
             raise ValueError("I have no str representation.")
+
         def __repr__(self):
             raise ValueError("I have no repr representation.")
 
@@ -134,8 +132,8 @@ def test_worker_bad_args(c, s, a, b):
     if sys.version_info[0] >= 3:
         tb = yield y._traceback()
         assert any('1 / 0' in line
-                  for line in pluck(3, traceback.extract_tb(tb))
-                  if line)
+                   for line in pluck(3, traceback.extract_tb(tb))
+                   if line)
     assert "Compute Failed" in hdlr.messages['warning'][0]
     logger.setLevel(old_level)
 
@@ -149,26 +147,6 @@ def test_worker_bad_args(c, s, a, b):
     assert tuple(results) == (3, 7)
 
 
-@gen_cluster()
-def dont_test_workers_update_center(s, a, b):
-    aa = rpc(ip=a.ip, port=a.port)
-
-    response = yield aa.update_data(data={'x': dumps(1), 'y': dumps(2)})
-    assert response['status'] == 'OK'
-    assert response['nbytes'] == {'x': sizeof(1), 'y': sizeof(2)}
-
-    assert a.data == {'x': 1, 'y': 2}
-    assert s.who_has == {'x': {a.address},
-                         'y': {a.address}}
-    assert s.has_what[a.address] == {'x', 'y'}
-
-    yield aa.delete_data(keys=['x'], close=True)
-    assert not s.who_has['x']
-    assert all('x' not in s for s in c.has_what.values())
-
-    aa.close_rpc()
-
-
 @slow
 @gen_cluster()
 def dont_test_delete_data_with_missing_worker(c, a, b):
@@ -294,6 +272,7 @@ def test_error_message():
     class MyException(Exception):
         def __init__(self, a, b):
             self.args = (a + b,)
+
         def __str__(self):
             return "MyException(%s)" % self.args
 
@@ -324,7 +303,7 @@ def test_io_loop(loop):
 @gen_cluster(client=True, ncores=[])
 def test_spill_to_disk(e, s):
     np = pytest.importorskip('numpy')
-    w = Worker(s.address, loop=s.loop, memory_limit=1000)
+    w = Worker(s.address, loop=s.loop, memory_limit=1200)
     yield w._start()
 
     x = e.submit(np.random.randint, 0, 255, size=500, dtype='u1', key='x')
@@ -397,13 +376,15 @@ def test_Executor(c, s):
         yield w._close()
 
 
+@pytest.mark.skip(reason="Leaks a large amount of memory")
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1)], timeout=30)
 def test_spill_by_default(c, s, w):
     da = pytest.importorskip('dask.array')
     x = da.ones(int(TOTAL_MEMORY * 0.7), chunks=10000000, dtype='u1')
     y = c.persist(x)
-    yield _wait(y)
+    yield wait(y)
     assert len(w.data.slow)  # something is on disk
+    del x, y
 
 
 @gen_cluster(ncores=[('127.0.0.1', 1)],
@@ -507,7 +488,6 @@ def test_system_monitor(s, a, b):
     b.monitor.update()
 
 
-
 @gen_cluster(client=True, ncores=[('127.0.0.1', 2, {'resources': {'A': 1}}),
                                   ('127.0.0.1', 1)])
 def test_restrictions(c, s, a, b):
@@ -709,9 +689,9 @@ def test_priorities_2(c, s, w):
     yield _wait(futures)
 
     log = [t[0] for t in w.log
-                if t[1] == 'executing'
-                and t[2] == 'memory'
-                and not t[0].startswith('finalize')]
+           if t[1] == 'executing'
+           and t[2] == 'memory'
+           and not t[0].startswith('finalize')]
 
     assert any(key.startswith('b1') for key in log[:len(log) // 2])
 
@@ -778,6 +758,7 @@ def test_fail_write_to_disk(c, s, a, b):
 def test_fail_write_many_to_disk(c, s, a, b):
     a.validate = False
     b.validate = False
+
     class Bad(object):
         def __init__(self, x):
             pass
@@ -801,3 +782,119 @@ def test_fail_write_many_to_disk(c, s, a, b):
     assert result == 2
     result = yield c.submit(inc, 2, workers=b.address)
     assert result == 3
+
+
+@gen_cluster()
+def test_pid(s, a, b):
+    assert s.worker_info[a.address]['pid'] == os.getpid()
+
+
+@gen_cluster(client=True)
+def test_get_client(c, s, a, b):
+    def f(x):
+        cc = get_client()
+        future = cc.submit(inc, x)
+        return future.result()
+
+    assert default_client() is c
+
+    future = c.submit(f, 10, workers=a.address)
+    result = yield future
+    assert result == 11
+
+    assert a._client
+    assert not b._client
+
+    assert a._client is c
+    assert default_client() is c
+
+    a_client = a._client
+
+    for i in range(10):
+        yield wait(c.submit(f, i))
+
+    assert a._client is a_client
+
+
+def test_get_client_sync(loop):
+    with cluster() as (s, [a, b]):
+        with Client(s['address'], loop=loop) as c:
+            def f(x):
+                cc = get_client()
+                future = cc.submit(inc, x)
+                return future.result()
+
+            future = c.submit(f, 10)
+            assert future.result() == 11
+
+
+@gen_cluster(client=True)
+def test_get_client_coroutine(c, s, a, b):
+    @gen.coroutine
+    def f():
+        client = yield get_client()
+        future = client.submit(inc, 10)
+        result = yield future
+        raise gen.Return(result)
+
+    results = yield c.run_coroutine(f)
+    assert results == {a.address: 11,
+                       b.address: 11}
+
+
+def test_get_client_coroutine_sync(loop):
+    with cluster() as (s, [a, b]):
+        with Client(s['address'], loop=loop) as c:
+            @gen.coroutine
+            def f():
+                client = yield get_client()
+                future = client.submit(inc, 10)
+                result = yield future
+                raise gen.Return(result)
+
+            results = c.run_coroutine(f)
+            assert results == {a['address']: 11,
+                               b['address']: 11}
+
+
+@gen_cluster()
+def test_global_workers(s, a, b):
+    from distributed.worker import _global_workers
+    n = len(_global_workers)
+    w = _global_workers[-1]()
+    assert w is a or w is b
+    yield a._close()
+    yield b._close()
+    assert len(_global_workers) == n - 2
+
+
+@pytest.mark.skipif(WINDOWS, reason="file descriptors")
+@gen_cluster(ncores=[])
+def test_worker_fds(s):
+    psutil = pytest.importorskip('psutil')
+    start = psutil.Process().num_fds()
+
+    worker = Worker(s.address, loop=s.loop)
+    yield worker._start()
+    middle = psutil.Process().num_fds()
+    assert middle > start
+
+    yield worker._close()
+
+    start = time()
+    while psutil.Process().num_fds() > start:
+        yield gen.sleep(0.01)
+        assert time() < start + 0.5
+
+
+@gen_cluster(ncores=[])
+def test_service_hosts_match_worker(s):
+    from distributed.http.worker import HTTPWorker
+    services = {('http', 0): HTTPWorker}
+    for host in ['tcp://0.0.0.0', 'tcp://127.0.0.2']:
+        w = Worker(s.address, services=services)
+        yield w._start(host)
+
+        sock = first(w.services['http']._sockets.values())
+        assert sock.getsockname()[0] == host.split('://')[1]
+        yield w._close()
