@@ -1,7 +1,6 @@
 from __future__ import print_function, division, absolute_import
 
 from concurrent.futures import CancelledError
-from datetime import timedelta
 from operator import add
 import os
 from time import sleep
@@ -12,26 +11,25 @@ from tornado import gen
 
 from dask import delayed
 from distributed import Client, Nanny, wait
-from distributed.compatibility import PY3
+from distributed.comm import CommClosedError
 from distributed.client import _wait
 from distributed.metrics import time
-from distributed.nanny import isalive
 from distributed.utils import sync, ignoring
-from distributed.utils_test import (gen_cluster, cluster, inc, loop, slow, div,
-        slowinc, slowadd)
+from distributed.utils_test import (gen_cluster, cluster, inc, slow, div,
+                                    slowinc, slowadd, captured_logger)
+from distributed.utils_test import loop # flake8: noqa
 
 
 def test_submit_after_failed_worker_sync(loop):
-    with cluster() as (s, [a, b]):
+    with cluster(active_rpc_timeout=10) as (s, [a, b]):
         with Client(s['address'], loop=loop) as c:
             L = c.map(inc, range(10))
             wait(L)
-            a['proc'].terminate()
+            a['proc']().terminate()
             total = c.submit(sum, L)
             assert total.result() == sum(map(inc, range(10)))
 
 
-
 @gen_cluster(client=True, timeout=60, active_rpc_timeout=10)
 def test_submit_after_failed_worker_async(c, s, a, b):
     n = Nanny(s.ip, s.port, ncores=2, loop=s.loop)
@@ -42,7 +40,7 @@ def test_submit_after_failed_worker_async(c, s, a, b):
     L = c.map(inc, range(10))
     yield _wait(L)
 
-    s.loop.add_callback(n._kill)
+    s.loop.add_callback(n.kill)
     total = c.submit(sum, L)
     result = yield total
     assert result == sum(map(inc, range(10)))
@@ -62,29 +60,29 @@ def test_submit_after_failed_worker(c, s, a, b):
 
 
 def test_gather_after_failed_worker(loop):
-    with cluster() as (s, [a, b]):
+    with cluster(active_rpc_timeout=10) as (s, [a, b]):
         with Client(s['address'], loop=loop) as c:
             L = c.map(inc, range(10))
             wait(L)
-            a['proc'].terminate()
+            a['proc']().terminate()
             result = c.gather(L)
             assert result == list(map(inc, range(10)))
 
 
 @slow
 def test_gather_then_submit_after_failed_workers(loop):
-    with cluster(nworkers=4) as (s, [w, x, y, z]):
+    with cluster(nworkers=4, active_rpc_timeout=10) as (s, [w, x, y, z]):
         with Client(s['address'], loop=loop) as c:
             L = c.map(inc, range(20))
             wait(L)
-            w['proc'].terminate()
+            w['proc']().terminate()
             total = c.submit(sum, L)
             wait([total])
 
             addr = c.who_has()[total.key][0]
             for d in [x, y, z]:
                 if d['address'] == addr:
-                    d['proc'].terminate()
+                    d['proc']().terminate()
                     break
             else:
                 assert 0, "Could not find worker %r" % (addr,)
@@ -98,10 +96,11 @@ def test_failed_worker_without_warning(c, s, a, b):
     L = c.map(inc, range(10))
     yield _wait(L)
 
-    original_process = a.process
-    a.process.terminate()
+    original_pid = a.pid
+    with ignoring(CommClosedError):
+        yield c._run(os._exit, 1, workers=[a.worker_address])
     start = time()
-    while a.process is original_process and not isalive(a.process):
+    while a.pid == original_pid:
         yield gen.sleep(0.01)
         assert time() - start < 10
 
@@ -167,10 +166,10 @@ def test_restart_cleared(c, s, a, b):
     yield c._restart()
 
     for coll in [s.tasks, s.dependencies, s.dependents, s.waiting,
-            s.waiting_data, s.who_has, s.host_restrictions,
-            s.worker_restrictions, s.loose_restrictions,
-            s.released, s.priority, s.exceptions, s.who_wants,
-            s.exceptions_blame]:
+                 s.waiting_data, s.who_has, s.host_restrictions,
+                 s.worker_restrictions, s.loose_restrictions,
+                 s.released, s.priority, s.exceptions, s.who_wants,
+                 s.exceptions_blame]:
         assert not coll
 
 
@@ -268,13 +267,14 @@ def test_multiple_clients_restart(s, a, b):
     assert x.cancelled()
     assert y.cancelled()
 
-    yield e1._shutdown(fast=True)
-    yield e2._shutdown(fast=True)
+    yield e1._close(fast=True)
+    yield e2._close(fast=True)
 
 
 @gen_cluster(Worker=Nanny)
 def test_restart_scheduler(s, a, b):
-    import gc; gc.collect()
+    import gc
+    gc.collect()
     addrs = (a.worker_address, b.worker_address)
     yield s.restart()
     assert len(s.ncores) == 2
@@ -289,7 +289,8 @@ def test_forgotten_futures_dont_clean_up_new_futures(c, s, a, b):
     yield c._restart()
     y = c.submit(inc, 1)
     del x
-    import gc; gc.collect()
+    import gc
+    gc.collect()
     yield gen.sleep(0.1)
     yield y
 
@@ -311,13 +312,13 @@ def test_broken_worker_during_computation(c, s, a, b):
 
     from random import random
     yield gen.sleep(random() / 2)
-    with ignoring(OSError):
-        n.process.terminate()
+    with ignoring(CommClosedError):  # comm will be closed abrupty
+        yield c._run(os._exit, 1, workers=[n.worker_address])
     yield gen.sleep(random() / 2)
-    with ignoring(OSError):
-        n.process.terminate()
+    with ignoring(CommClosedError, EnvironmentError):  # perhaps new worker can't be contacted yet
+        yield c._run(os._exit, 1, workers=[n.worker_address])
 
-    result = yield c._gather(L)
+    result = yield c.gather(L)
     assert isinstance(result[0], int)
 
     yield n._close()
@@ -358,7 +359,8 @@ def test_worker_who_has_clears_after_failed_connection(c, s, a, b):
         a.release_dep(dep, report=True)
 
     n_worker_address = n.worker_address
-    n.process.terminate()
+    with ignoring(CommClosedError):
+        yield c._run(os._exit, 1, workers=[n_worker_address])
 
     while len(s.workers) > 2:
         yield gen.sleep(0.01)
@@ -370,3 +372,15 @@ def test_worker_who_has_clears_after_failed_connection(c, s, a, b):
     assert not any(n_worker_address in s for s in a.who_has.values())
 
     yield n._close()
+
+
+@slow
+@gen_cluster(client=True, timeout=60, Worker=Nanny, ncores=[('127.0.0.1', 1)])
+def test_restart_timeout_on_long_running_task(c, s, a):
+    with captured_logger('distributed.scheduler') as sio:
+        future = c.submit(sleep, 3600)
+        yield gen.sleep(0.1)
+        yield c.restart(timeout=20)
+
+    text = sio.getvalue()
+    assert 'timeout' not in text.lower()
