@@ -40,13 +40,11 @@ from .sizeof import safe_sizeof as sizeof
 from .threadpoolexecutor import ThreadPoolExecutor, secede as tpe_secede
 from .utils import (funcname, get_ip, has_arg, _maybe_complex, log_errors,
                     ignoring, validate_key, mp_context, import_file,
-                    silence_logging)
+                    silence_logging, thread_state)
 from .utils_comm import pack_data, gather_from_workers
 
 _ncores = mp_context.cpu_count()
 
-thread_state = threading.local()
-
 logger = logging.getLogger(__name__)
 
 LOG_PDB = config.get('pdb-on-err') or os.environ.get('DASK_ERROR_PDB', False)
@@ -67,6 +65,7 @@ PENDING = ('waiting', 'ready', 'constrained')
 PROCESSING = ('waiting', 'ready', 'constrained', 'executing', 'long-running')
 READY = ('ready', 'constrained')
 
+
 _global_workers = []
 
 
@@ -76,7 +75,7 @@ class WorkerBase(ServerNode):
                  name=None, heartbeat_interval=5000, reconnect=True,
                  memory_limit='auto', executor=None, resources=None,
                  silence_logs=None, death_timeout=None, preload=(),
-                 security=None, **kwargs):
+                 security=None, contact_address=None, **kwargs):
         if scheduler_port is None:
             scheduler_addr = coerce_to_address(scheduler_ip)
         else:
@@ -87,6 +86,7 @@ class WorkerBase(ServerNode):
         self.available_resources = (resources or {}).copy()
         self.death_timeout = death_timeout
         self.preload = preload
+        self.contact_address = contact_address
         if silence_logs:
             silence_logging(level=silence_logs)
 
@@ -143,28 +143,30 @@ class WorkerBase(ServerNode):
         self.service_specs = services or {}
 
         handlers = {
-          'gather': self.gather,
-          'compute-stream': self.compute_stream,
-          'run': self.run,
-          'run_coroutine': self.run_coroutine,
-          'get_data': self.get_data,
-          'update_data': self.update_data,
-          'delete_data': self.delete_data,
-          'terminate': self.terminate,
-          'ping': pingpong,
-          'health': self.host_health,
-          'upload_file': self.upload_file,
-          'start_ipython': self.start_ipython,
-          'keys': self.keys,
+            'gather': self.gather,
+            'compute-stream': self.compute_stream,
+            'run': self.run,
+            'run_coroutine': self.run_coroutine,
+            'get_data': self.get_data,
+            'update_data': self.update_data,
+            'delete_data': self.delete_data,
+            'terminate': self.terminate,
+            'ping': pingpong,
+            'health': self.host_health,
+            'upload_file': self.upload_file,
+            'start_ipython': self.start_ipython,
+            'keys': self.keys,
         }
 
         super(WorkerBase, self).__init__(handlers, io_loop=self.loop,
                                          connection_args=self.connection_args,
                                          **kwargs)
 
-        self.heartbeat_callback = PeriodicCallback(self.heartbeat,
-                                                   self.heartbeat_interval,
-                                                   io_loop=self.loop)
+        pc = PeriodicCallback(self.heartbeat,
+                              self.heartbeat_interval,
+                              io_loop=self.loop)
+        self.periodic_callbacks['heartbeat'] = pc
+        self._address = contact_address
 
     @property
     def worker_address(self):
@@ -186,18 +188,18 @@ class WorkerBase(ServerNode):
                     kwargs = {}
 
                 yield self.scheduler.register(
-                        address=self.address,
-                        name=self.name,
-                        ncores=self.ncores,
-                        now=time(),
-                        host_info=self.host_health(),
-                        services=self.service_ports,
-                        memory_limit=self.memory_limit,
-                        executing=len(self.executing),
-                        in_memory=len(self.data),
-                        ready=len(self.ready),
-                        in_flight=len(self.in_flight_tasks),
-                        **kwargs)
+                    address=self.contact_address,
+                    name=self.name,
+                    ncores=self.ncores,
+                    now=time(),
+                    host_info=self.host_health(),
+                    services=self.service_ports,
+                    memory_limit=self.memory_limit,
+                    executing=len(self.executing),
+                    in_memory=len(self.data),
+                    ready=len(self.ready),
+                    in_flight=len(self.in_flight_tasks),
+                    **kwargs)
             finally:
                 self.heartbeat_active = False
         else:
@@ -205,8 +207,10 @@ class WorkerBase(ServerNode):
 
     @gen.coroutine
     def _register_with_scheduler(self):
-        self.heartbeat_callback.stop()
+        self.periodic_callbacks['heartbeat'].stop()
         start = time()
+        if self.contact_address is None:
+            self.contact_address = self.address
         while True:
             if self.death_timeout and time() > start + self.death_timeout:
                 yield self._close(timeout=1)
@@ -215,7 +219,8 @@ class WorkerBase(ServerNode):
                 raise gen.Return
             try:
                 future = self.scheduler.register(
-                        ncores=self.ncores, address=self.address,
+                        ncores=self.ncores,
+                        address=self.contact_address,
                         keys=list(self.data),
                         name=self.name,
                         nbytes=self.nbytes,
@@ -241,7 +246,7 @@ class WorkerBase(ServerNode):
                 pass
         if resp != 'OK':
             raise ValueError("Unexpected response from register: %r" % (resp,))
-        self.heartbeat_callback.start()
+        self.periodic_callbacks['heartbeat'].start()
 
     def start_services(self, listen_ip=''):
         for k, v in self.service_specs.items():
@@ -254,7 +259,6 @@ class WorkerBase(ServerNode):
                 v, kwargs = v
             else:
                 v, kwargs = v, {}
-
             self.services[k] = v(self, io_loop=self.loop, **kwargs)
             self.services[k].listen((listen_ip, port))
             self.service_ports[k] = self.services[k].port
@@ -266,32 +270,45 @@ class WorkerBase(ServerNode):
         # XXX Factor this out
         if not addr_or_port:
             # Default address is the required one to reach the scheduler
+            listen_host = get_address_host(self.scheduler.address)
             self.listen(get_local_address_for(self.scheduler.address),
                         listen_args=self.listen_args)
             self.ip = get_address_host(self.address)
         elif isinstance(addr_or_port, int):
             # addr_or_port is an integer => assume TCP
-            self.ip = get_ip(
+            listen_host = self.ip = get_ip(
                 get_address_host(self.scheduler.address)
             )
-            self.listen((self.ip, addr_or_port),
+            self.listen((listen_host, addr_or_port),
                         listen_args=self.listen_args)
         else:
             self.listen(addr_or_port, listen_args=self.listen_args)
             self.ip = get_address_host(self.address)
+            try:
+                listen_host = get_address_host(addr_or_port)
+            except ValueError:
+                listen_host = addr_or_port
+
+        if '://' in listen_host:
+            protocol, listen_host = listen_host.split('://')
 
         self.name = self.name or self.address
         preload_modules(self.preload, parameter=self, file_dir=self.local_dir)
         # Services listen on all addresses
         # Note Nanny is not a "real" service, just some metadata
         # passed in service_ports...
-        self.start_services()
+        self.start_services(listen_host)
+
+        try:
+            listening_address = '%s%s:%d' % (self.listener.prefix, listen_host, self.port)
+        except Exception:
+            listening_address = '%s%s' % (self.listener.prefix, listen_host)
 
         logger.info('      Start worker at: %26s', self.address)
+        logger.info('         Listening to: %26s', listening_address)
         for k, v in self.service_ports.items():
-            logger.info('  %16s at: %20s:%d' % (k, self.ip, v))
-        logger.info('Waiting to connect to: %26s',
-                    self.scheduler.address)
+            logger.info('  %16s at: %26s' % (k, listen_host + ':' + str(v)))
+        logger.info('Waiting to connect to: %26s', self.scheduler.address)
         logger.info('-' * 49)
         logger.info('              Threads: %26d', self.ncores)
         if self.memory_limit:
@@ -302,7 +319,7 @@ class WorkerBase(ServerNode):
         yield self._register_with_scheduler()
 
         if self.status == 'running':
-            logger.info('        Registered to: %32s', self.scheduler.address)
+            logger.info('        Registered to: %26s', self.scheduler.address)
             logger.info('-' * 49)
 
     def start(self, port=0):
@@ -320,18 +337,20 @@ class WorkerBase(ServerNode):
         if self.status in ('closed', 'closing'):
             return
         logger.info("Stopping worker at %s", self.address)
-        if self._client:
-            yield self._client._close()
         self.status = 'closing'
         self.stop()
-        self.heartbeat_callback.stop()
+        for pc in self.periodic_callbacks.values():
+            pc.stop()
         with ignoring(EnvironmentError, gen.TimeoutError):
             if report:
                 yield gen.with_timeout(timedelta(seconds=timeout),
-                        self.scheduler.unregister(address=self.address),
-                        io_loop=self.loop)
+                                       self.scheduler.unregister(address=self.contact_address),
+                                       io_loop=self.loop)
         self.scheduler.close_rpc()
-        self.executor.shutdown()
+        if isinstance(self.executor, ThreadPoolExecutor):
+            self.executor.shutdown(timeout=timeout)
+        else:
+            self.executor.shutdown(wait=False)
         if os.path.exists(self.local_dir):
             shutil.rmtree(self.local_dir)
 
@@ -347,6 +366,7 @@ class WorkerBase(ServerNode):
         self.rpc.close()
         self._closed.set()
         self._remove_from_global_workers()
+        yield super(WorkerBase, self).close()
 
     def __del__(self):
         self._remove_from_global_workers()
@@ -381,7 +401,7 @@ class WorkerBase(ServerNode):
         # logger.info("%s:%d Starts job %d, %s", self.ip, self.port, i, key)
         future = self.executor.submit(function, *args, **kwargs)
         pc = PeriodicCallback(lambda: logger.debug("future state: %s - %s",
-            key, future._state), 1000, io_loop=self.loop); pc.start()
+                                                   key, future._state), 1000, io_loop=self.loop); pc.start()
         try:
             yield future
         finally:
@@ -438,7 +458,7 @@ class WorkerBase(ServerNode):
             if report:
                 logger.debug("Reporting loss of keys to scheduler")
                 # TODO: this route seems to not exist?
-                yield self.scheduler.remove_keys(address=self.address,
+                yield self.scheduler.remove_keys(address=self.contact_address,
                                                  keys=list(keys))
         raise Return('OK')
 
@@ -490,7 +510,7 @@ class WorkerBase(ServerNode):
             self.total_resources[r] = quantity
 
         yield self.scheduler.set_resources(resources=self.total_resources,
-                                           worker=self.address)
+                                           worker=self.contact_address)
 
     def start_ipython(self, comm):
         """Start an IPython kernel
@@ -565,10 +585,10 @@ class WorkerBase(ServerNode):
     @gen.coroutine
     def gather(self, comm=None, who_has=None):
         who_has = {k: [coerce_to_address(addr) for addr in v]
-                    for k, v in who_has.items()
-                    if k not in self.data}
+                   for k, v in who_has.items()
+                   if k not in self.data}
         result, missing_keys, missing_workers = yield gather_from_workers(
-                who_has, rpc=self.rpc)
+            who_has, rpc=self.rpc)
         if missing_keys:
             logger.warning("Could not find data: %s on workers: %s (who_has: %s)",
                            missing_keys, missing_workers, who_has)
@@ -624,7 +644,10 @@ def dumps_function(func):
     """ Dump a function to bytes, cache functions """
     if func not in cache:
         b = pickle.dumps(func)
-        cache[func] = b
+        if len(b) < 100000:
+            cache[func] = b
+        else:
+            return b
     return cache[func]
 
 
@@ -657,7 +680,7 @@ def dumps_task(task):
             return d
         elif not any(map(_maybe_complex, task[1:])):
             return {'function': dumps_function(task[0]),
-                        'args': pickle.dumps(task[1:])}
+                    'args': pickle.dumps(task[1:])}
     return to_serialize(task)
 
 
@@ -721,7 +744,7 @@ def convert_args_to_str(args, max_len=None):
     for i, arg in enumerate(args):
         try:
             sarg = repr(arg)
-        except:
+        except Exception:
             sarg = "< could not convert arg to str >"
         strs[i] = sarg
         length += len(sarg) + 2
@@ -740,7 +763,7 @@ def convert_kwargs_to_str(kwargs, max_len=None):
     for i, (argname, arg) in enumerate(kwargs.items()):
         try:
             sarg = repr(arg)
-        except:
+        except Exception:
             sarg = "< could not convert arg to str >"
         skwarg = repr(argname) + ": " + sarg
         strs[i] = skwarg
@@ -774,12 +797,12 @@ def run(server, comm, function, args=(), kwargs={}, is_coro=False, wait=True):
             result = (yield result) if wait else None
     except Exception as e:
         logger.warning(" Run Failed\n"
-            "Function: %s\n"
-            "args:     %s\n"
-            "kwargs:   %s\n",
-            str(funcname(function))[:1000],
-            convert_args_to_str(args, max_len=1000),
-            convert_kwargs_to_str(kwargs, max_len=1000), exc_info=True)
+                       "Function: %s\n"
+                       "args:     %s\n"
+                       "kwargs:   %s\n",
+                       str(funcname(function))[:1000],
+                       convert_args_to_str(args, max_len=1000),
+                       convert_kwargs_to_str(kwargs, max_len=1000), exc_info=True)
 
         response = error_message(e)
     else:
@@ -950,6 +973,7 @@ class Worker(WorkerBase):
     distributed.scheduler.Scheduler
     distributed.nanny.Nanny
     """
+
     def __init__(self, *args, **kwargs):
         self.tasks = dict()
         self.task_state = dict()
@@ -999,24 +1023,24 @@ class Worker(WorkerBase):
         self.validate = kwargs.pop('validate', False)
 
         self._transitions = {
-                ('waiting', 'ready'): self.transition_waiting_ready,
-                ('waiting', 'memory'): self.transition_waiting_done,
-                ('waiting', 'error'): self.transition_waiting_done,
-                ('ready', 'executing'): self.transition_ready_executing,
-                ('ready', 'memory'): self.transition_ready_memory,
-                ('constrained', 'executing'): self.transition_constrained_executing,
-                ('executing', 'memory'): self.transition_executing_done,
-                ('executing', 'error'): self.transition_executing_done,
-                ('executing', 'long-running'): self.transition_executing_long_running,
-                ('long-running', 'error'): self.transition_executing_done,
-                ('long-running', 'memory'): self.transition_executing_done,
+            ('waiting', 'ready'): self.transition_waiting_ready,
+            ('waiting', 'memory'): self.transition_waiting_done,
+            ('waiting', 'error'): self.transition_waiting_done,
+            ('ready', 'executing'): self.transition_ready_executing,
+            ('ready', 'memory'): self.transition_ready_memory,
+            ('constrained', 'executing'): self.transition_constrained_executing,
+            ('executing', 'memory'): self.transition_executing_done,
+            ('executing', 'error'): self.transition_executing_done,
+            ('executing', 'long-running'): self.transition_executing_long_running,
+            ('long-running', 'error'): self.transition_executing_done,
+            ('long-running', 'memory'): self.transition_executing_done,
         }
 
         self._dep_transitions = {
-                ('waiting', 'flight'): self.transition_dep_waiting_flight,
-                ('waiting', 'memory'): self.transition_dep_waiting_memory,
-                ('flight', 'waiting'): self.transition_dep_flight_waiting,
-                ('flight', 'memory'): self.transition_dep_flight_memory,
+            ('waiting', 'flight'): self.transition_dep_waiting_flight,
+            ('waiting', 'memory'): self.transition_dep_waiting_memory,
+            ('flight', 'waiting'): self.transition_dep_flight_waiting,
+            ('flight', 'memory'): self.transition_dep_flight_memory,
         }
 
         self.incoming_transfer_log = deque(maxlen=(100000))
@@ -1031,10 +1055,10 @@ class Worker(WorkerBase):
 
     def __str__(self):
         return "<%s: %s, %s, stored: %d, running: %d/%d, ready: %d, comm: %d, waiting: %d>" % (
-                self.__class__.__name__, self.address, self.status,
-                len(self.data), len(self.executing), self.ncores,
-                len(self.ready), len(self.in_flight_tasks),
-                len(self.waiting_for_data))
+            self.__class__.__name__, self.address, self.status,
+            len(self.data), len(self.executing), self.ncores,
+            len(self.ready), len(self.in_flight_tasks),
+            len(self.waiting_for_data))
 
     __repr__ = __str__
 
@@ -1065,6 +1089,11 @@ class Worker(WorkerBase):
                     break
                 except EnvironmentError as e:
                     break
+                except Exception as e:
+                    logger.error("Worker failed to read message. "
+                                 "This will likely cause the cluster to fail.",
+                                 exc_info=True)
+                    raise
 
                 start = time()
 
@@ -1080,7 +1109,7 @@ class Worker(WorkerBase):
                     elif op == 'compute-task':
                         self.add_task(**msg)
                     elif op == 'release-task':
-                        self.log.append((msg['key'], 'release-task'))
+                        self.log.append((msg['key'], 'release-task', msg.get('reason')))
                         self.release_key(report=False, **msg)
                     elif op == 'delete-data':
                         self.delete_data(**msg)
@@ -1103,8 +1132,8 @@ class Worker(WorkerBase):
             raise
 
     def add_task(self, key, function=None, args=None, kwargs=None, task=None,
-            who_has=None, nbytes=None, priority=None, duration=None,
-            resource_restrictions=None, **kwargs2):
+                 who_has=None, nbytes=None, priority=None, duration=None,
+                 resource_restrictions=None, **kwargs2):
         if isinstance(priority, list):
             priority.insert(1, self.priority_counter)
         try:
@@ -1113,7 +1142,7 @@ class Worker(WorkerBase):
                 if state in ('memory', 'error'):
                     if state == 'memory':
                         assert key in self.data
-                    logger.debug("Asked to compute pre-existing result: %s: %s" ,
+                    logger.debug("Asked to compute pre-existing result: %s: %s",
                                  key, state)
                     self.send_task_state_to_scheduler(key)
                     return
@@ -1166,9 +1195,11 @@ class Worker(WorkerBase):
 
                 if dep not in self.dep_state:
                     if self.task_state.get(dep) == 'memory':
-                        self.dep_state[dep] = 'memory'
+                        state = 'memory'
                     else:
-                        self.dep_state[dep] = 'waiting'
+                        state = 'waiting'
+                    self.dep_state[dep] = state
+                    self.log.append((dep, 'new-dep', state))
 
                 if self.dep_state[dep] != 'memory':
                     self.waiting_for_data[key].add(dep)
@@ -1198,7 +1229,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     ###############
@@ -1230,7 +1262,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def transition_dep_flight_waiting(self, dep, worker=None):
@@ -1261,7 +1294,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def transition_dep_flight_memory(self, dep, value=None):
@@ -1270,12 +1304,19 @@ class Worker(WorkerBase):
                 assert dep in self.in_flight_tasks
 
             del self.in_flight_tasks[dep]
-            self.dep_state[dep] = 'memory'
-            self.put_key_in_memory(dep, value)
+            if self.dependents[dep]:
+                self.dep_state[dep] = 'memory'
+                self.put_key_in_memory(dep, value)
+                self.batched_stream.send({'op': 'add-keys',
+                                          'keys': [dep]})
+            else:
+                self.release_dep(dep)
+
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def transition_dep_waiting_memory(self, dep, value=None):
@@ -1288,11 +1329,13 @@ class Worker(WorkerBase):
                     assert self.task_state[dep] == 'memory'
                 except Exception as e:
                     logger.exception(e)
-                    import pdb; pdb.set_trace()
+                    import pdb
+                    pdb.set_trace()
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def transition(self, key, finish, **kwargs):
@@ -1325,7 +1368,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def transition_waiting_done(self, key, value=None):
@@ -1341,7 +1385,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def transition_ready_executing(self, key):
@@ -1358,7 +1403,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def transition_ready_memory(self, key, value=None):
@@ -1417,7 +1463,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def transition_executing_long_running(self, key, compute_duration=None):
@@ -1435,7 +1482,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def maybe_transition_long_running(self, key, compute_duration=None):
@@ -1479,7 +1527,7 @@ class Worker(WorkerBase):
                 if missing_deps:
                     logger.info("Can't find dependencies for key %s", key)
                     missing_deps2 = {dep for dep in missing_deps
-                                         if dep not in self._missing_dep_flight}
+                                     if dep not in self._missing_dep_flight}
                     for dep in missing_deps2:
                         self._missing_dep_flight.add(dep)
                     self.loop.add_callback(self.handle_missing_dep,
@@ -1499,7 +1547,7 @@ class Worker(WorkerBase):
                     if dep not in self.who_has:
                         continue
                     workers = [w for w in self.who_has[dep]
-                                  if w not in self.in_flight_workers]
+                               if w not in self.in_flight_workers]
                     if not workers:
                         in_flight = True
                         continue
@@ -1510,7 +1558,7 @@ class Worker(WorkerBase):
                     for d in to_gather:
                         self.transition_dep(d, 'flight', worker=worker)
                     self.loop.add_callback(self.gather_dep, worker, dep,
-                            to_gather, total_nbytes, cause=key)
+                                           to_gather, total_nbytes, cause=key)
                     changed = True
 
                 if not deps and not in_flight:
@@ -1518,7 +1566,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def send_task_state_to_scheduler(self, key):
@@ -1566,9 +1615,6 @@ class Worker(WorkerBase):
         if key not in self.nbytes:
             self.nbytes[key] = sizeof(value)
 
-        if key not in self.types:
-            self.types[key] = type(value)
-
         self.types[key] = type(value)
 
         for dep in self.dependents.get(key, ()):
@@ -1610,10 +1656,13 @@ class Worker(WorkerBase):
                 if self.validate:
                     self.validate_state()
 
+                deps = tuple(dep for dep in deps
+                                 if self.dep_state.get(dep) in ('waiting', 'flight'))
+
                 self.log.append(('request-dep', dep, worker, deps))
                 logger.debug("Request %d keys", len(deps))
                 start = time()
-                response = yield self.rpc(worker).get_data(keys=list(deps),
+                response = yield self.rpc(worker).get_data(keys=deps,
                                                            who=self.address)
                 stop = time()
 
@@ -1655,7 +1704,8 @@ class Worker(WorkerBase):
             except Exception as e:
                 logger.exception(e)
                 if self.batched_stream and LOG_PDB:
-                    import pdb; pdb.set_trace()
+                    import pdb
+                    pdb.set_trace()
                 raise
             finally:
                 self.comm_nbytes -= total_nbytes
@@ -1668,6 +1718,9 @@ class Worker(WorkerBase):
 
                     if d not in response and d in self.dependents:
                         self.log.append(('missing-dep', d))
+                        self.batched_stream.send({'op': 'missing-data',
+                                                  'errant_worker': worker,
+                                                  'key': d})
 
                 if self.validate:
                     self.validate_state()
@@ -1730,8 +1783,11 @@ class Worker(WorkerBase):
             else:
                 raise
         finally:
-            for dep in original_deps:
-                self._missing_dep_flight.remove(dep)
+            try:
+                for dep in original_deps:
+                    self._missing_dep_flight.remove(dep)
+            except KeyError:
+                pass
 
             self.ensure_communicating()
 
@@ -1757,7 +1813,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def release_key(self, key, cause=None, reason=None, report=True):
@@ -1769,7 +1826,7 @@ class Worker(WorkerBase):
                 self.task_state[key] = state
                 return
             if cause:
-                self.log.append((key, 'release-key', cause))
+                self.log.append((key, 'release-key', {'cause': cause}))
             else:
                 self.log.append((key, 'release-key'))
             del self.tasks[key]
@@ -1783,7 +1840,7 @@ class Worker(WorkerBase):
 
             for dep in self.dependencies.pop(key, ()):
                 self.dependents[dep].remove(key)
-                if not self.dependents[dep] and self.dep_state[dep] == 'waiting':
+                if not self.dependents[dep] and self.dep_state[dep] in ('waiting', 'flight'):
                     self.release_dep(dep)
 
             if key in self.threads:
@@ -1814,7 +1871,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def release_dep(self, dep, report=False):
@@ -1838,7 +1896,8 @@ class Worker(WorkerBase):
                 del self.nbytes[dep]
 
             if dep in self.in_flight_tasks:
-                del self.in_flight_tasks[dep]
+                worker = self.in_flight_tasks.pop(dep)
+                self.in_flight_workers[worker].remove(dep)
 
             for key in self.dependents.pop(dep, ()):
                 self.dependencies[key].remove(dep)
@@ -1851,7 +1910,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def rescind_key(self, key):
@@ -1878,7 +1938,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     ################
@@ -1913,7 +1974,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     @gen.coroutine
@@ -1929,8 +1991,9 @@ class Worker(WorkerBase):
             function, args, kwargs = self.tasks[key]
 
             start = time()
-            args2 = pack_data(args, self.data, key_types=str)
-            kwargs2 = pack_data(kwargs, self.data, key_types=str)
+            data = {k: self.data[k] for k in self.dependencies[key]}
+            args2 = pack_data(args, data, key_types=str)
+            kwargs2 = pack_data(kwargs, data, key_types=str)
             stop = time()
             if stop - start > 0.005:
                 self.startstops[key].append(('disk-read', start, stop))
@@ -1952,7 +2015,7 @@ class Worker(WorkerBase):
             result['key'] = key
             value = result.pop('result', None)
             self.startstops[key].append(('compute', result['start'],
-                                                    result['stop']))
+                                         result['stop']))
             self.threads[key] = result['thread']
 
             if result['op'] == 'task-finished':
@@ -1966,14 +2029,14 @@ class Worker(WorkerBase):
                 self.exceptions[key] = result['exception']
                 self.tracebacks[key] = result['traceback']
                 logger.warning(" Compute Failed\n"
-                    "Function:  %s\n"
-                    "args:      %s\n"
-                    "kwargs:    %s\n"
-                    "Exception: %s\n",
-                    str(funcname(function))[:1000],
-                    convert_args_to_str(args2, max_len=1000),
-                    convert_kwargs_to_str(kwargs2, max_len=1000),
-                    repr(pickle.loads(result['exception'])))
+                               "Function:  %s\n"
+                               "args:      %s\n"
+                               "kwargs:    %s\n"
+                               "Exception: %s\n",
+                               str(funcname(function))[:1000],
+                               convert_args_to_str(args2, max_len=1000),
+                               convert_kwargs_to_str(kwargs2, max_len=1000),
+                               repr(pickle.loads(result['exception'])))
                 self.transition(key, 'error')
 
             logger.debug("Send compute response to scheduler: %s, %s", key,
@@ -1991,7 +2054,8 @@ class Worker(WorkerBase):
             else:
                 logger.exception(e)
                 if LOG_PDB:
-                    import pdb; pdb.set_trace()
+                    import pdb
+                    pdb.set_trace()
                 raise
         finally:
             if key in self.executing:
@@ -2040,7 +2104,8 @@ class Worker(WorkerBase):
                 self.validate_key_executing(key)
         except Exception as e:
             logger.exception(e)
-            import pdb; pdb.set_trace()
+            import pdb
+            pdb.set_trace()
             raise
 
     def validate_dep_waiting(self, dep):
@@ -2076,7 +2141,8 @@ class Worker(WorkerBase):
                 raise ValueError("Unknown dependent state", state)
         except Exception as e:
             logger.exception(e)
-            import pdb; pdb.set_trace()
+            import pdb
+            pdb.set_trace()
             raise
 
     def validate_state(self):
@@ -2113,7 +2179,8 @@ class Worker(WorkerBase):
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
-                import pdb; pdb.set_trace()
+                import pdb
+                pdb.set_trace()
             raise
 
     def stateof(self, key):
@@ -2124,14 +2191,21 @@ class Worker(WorkerBase):
 
     def story(self, *keys):
         return [msg for msg in self.log
-                    if any(key in msg for key in keys)
-                    or any(key in c
-                           for key in keys
-                           for c in msg
-                           if isinstance(c, (tuple, list, set)))]
+                if any(key in msg for key in keys)
+                or any(key in c
+                       for key in keys
+                       for c in msg
+                       if isinstance(c, (tuple, list, set)))]
 
     @property
     def client(self):
+        with self._lock:
+            if self._client:
+                return self._client
+            else:
+                return self._get_client()
+
+    def _get_client(self, timeout=3):
         """ Get local client attached to this worker
 
         If no such client exists, create one
@@ -2140,29 +2214,27 @@ class Worker(WorkerBase):
         --------
         get_client
         """
-        with self._lock:
-            if self._client:
-                return self._client
+        try:
+            from .client import default_client
+            client = default_client()
+        except ValueError:  # no clients found, need to make a new one
+            pass
+        else:
+            if (client.scheduler and client.scheduler.address == self.scheduler.address
+                    or client._start_arg == self.scheduler.address):
+                self._client = client
 
-            try:
-                from .client import default_client
-                client = default_client()
-            except ValueError:  # no clients found, need to make a new one
-                pass
-            else:
-                if client.scheduler.address == self.scheduler.address:
-                    self._client = client
-
-            if not self._client:
-                from .client import Client
-                asynchronous = get_thread_identity() == self.loop._thread_ident
-                self._client = Client(self.scheduler.address, loop=self.loop,
-                                      security=self.security,
-                                      set_as_default=True,
-                                      asynchronous=asynchronous)
-                if not asynchronous:
-                    assert self._client.status == 'running'
-            return self._client
+        if not self._client:
+            from .client import Client
+            asynchronous = get_thread_identity() == self.loop._thread_ident
+            self._client = Client(self.scheduler.address, loop=self.loop,
+                                  security=self.security,
+                                  set_as_default=True,
+                                  asynchronous=asynchronous,
+                                  timeout=timeout)
+            if not asynchronous:
+                assert self._client.status == 'running'
+        return self._client
 
 
 def get_worker():
@@ -2193,7 +2265,7 @@ def get_worker():
         raise ValueError("No workers found")
 
 
-def get_client(address=None):
+def get_client(address=None, timeout=3):
     """ Get a client while within a task
 
     This client connects to the same scheduler to which the worker is connected
@@ -2222,7 +2294,7 @@ def get_client(address=None):
         pass
     else:
         if not address or worker.scheduler.address == address:
-            return worker.client
+            return worker._get_client(timeout=timeout)
 
     from .client import _get_global_client
     client = _get_global_client()  # TODO: assumes the same scheduler
@@ -2230,7 +2302,7 @@ def get_client(address=None):
         return client
     elif address:
         from .client import Client
-        return Client(address)
+        return Client(address, timeout=timeout)
     else:
         raise ValueError("No global client found and no address provided")
 
